@@ -1,7 +1,7 @@
 .self_intersection <- function(x, y) {
 
-  x <- methods::as(x, "dgCMatrix")
-  y <- methods::as(y, "dgCMatrix")
+  x <- convert_to_sparse(x)
+  y <- convert_to_sparse(y)
   self_intersection_C(x_i = x@i,
                       x_p = x@p,
                       y_i = y@i,
