@@ -19,3 +19,8 @@ NumericVector zadeh_I(double x, NumericVector y);
 void zadeh_I(double x, SparseVector *A);
 
 void intersect(SparseVector *A, SparseVector B);
+
+double tnorm_Zadeh(double x, double y);
+double implication_Zadeh(double x, double y);
+double tnorm_Lukasiewicz(double x, double y);
+double implication_Lukasiewicz(double x, double y);
