@@ -1,5 +1,6 @@
 #include <Rcpp.h>
 #include "set_operations_galois.h"
+#include "Logics.h"
 using namespace Rcpp;
 
 static void chkIntFn(void *dummy) {
@@ -17,7 +18,7 @@ bool checkInterrupt() {
 
 // Functions to compute the next pseudo-closed set
 
-void compute_direct_sum(SparseVector A,
+bool compute_direct_sum(SparseVector A,
                         int a_i,
                         double grade_i,
                         int imax,
@@ -26,11 +27,28 @@ void compute_direct_sum(SparseVector A,
   reinitVector(res);
 
   cloneVector(res, A);
+  bool can = true;
 
   int resp = res->i.used;
 
   for (size_t i = 0; i < A.i.used; i++) {
 
+    if (A.i.array[i] == a_i) {
+
+      if ((A.x.array[i] - grade_i) >= -1.e-3) {
+
+        // Rcout << "    -> NO: " << A.x.array[i] << " >= " << grade_i << "\n";
+
+        can = false;
+
+      } else {
+
+        // Rcout << "    -> SI: " << A.x.array[i] << " < " << grade_i << "\n";
+
+      }
+
+    }
+
     if (A.i.array[i] >= a_i) {
 
       resp = i;
@@ -46,6 +64,8 @@ void compute_direct_sum(SparseVector A,
   insertArray(&(res->i), a_i);
   insertArray(&(res->x), grade_i);
 
+  return can;
+
 }
 
 void semantic_closure(SparseVector A,
@@ -115,6 +135,70 @@ void semantic_closure(SparseVector A,
 
 }
 
+bool is_set_preceding2(SparseVector B,
+                      SparseVector C,
+                      int a_i,
+                      double grade_i) {
+
+  if (grade_i == 0 || grade_i == -1) {
+    return false;
+  }
+
+  size_t bi_lt_a_i_size = 0, ci_lt_a_i_size = 0;
+  size_t bi_idx = 0, ci_idx = 0;
+  double bx_at_a_i = 0.0, cx_at_a_i = 0.0;
+
+  // Find elements in B and C that are less than a_i
+  for (size_t i = 0; i < B.i.used && bi_idx < B.i.used; i++) {
+    if (B.i.array[i] < a_i) {
+      bi_lt_a_i_size++;
+    } else if (B.i.array[i] == a_i) {
+      bx_at_a_i = B.x.array[i];
+    } else {
+      break;
+    }
+    bi_idx++;
+  }
+
+  for (size_t i = 0; i < C.i.used && ci_idx < C.i.used; i++) {
+    if (C.i.array[i] < a_i) {
+      ci_lt_a_i_size++;
+    } else if (C.i.array[i] == a_i) {
+      cx_at_a_i = C.x.array[i];
+    } else {
+      break;
+    }
+    ci_idx++;
+  }
+
+  if (cx_at_a_i != grade_i) {
+    return false;
+  }
+
+  if (grade_i == 1 && bx_at_a_i == 1) {
+    return false;
+  }
+
+  if (grade_i == -1 && bx_at_a_i != -1) {
+    return false;
+  }
+
+  if (ci_lt_a_i_size != bi_lt_a_i_size) {
+    return false;
+  }
+
+  // Check that the elements less than a_i in B and C are the same
+  for (size_t i = 0; i < ci_lt_a_i_size; i++) {
+    if (C.i.array[i] != B.i.array[i] || C.x.array[i] != B.x.array[i]) {
+      return false;
+    }
+  }
+
+  return true;
+
+}
+
+
 bool is_set_preceding(SparseVector B,
                       SparseVector C,
                       int a_i,
@@ -165,13 +249,15 @@ bool is_set_preceding(SparseVector B,
 
   }
 
-  if (cx_at_a_i != grade_i) {
+  if (fabs(cx_at_a_i - grade_i) > 1.e-3) {
 
     freeArray(&cx_lt_a_i);
     freeArray(&bx_lt_a_i);
     freeArray(&ci_lt_a_i);
     freeArray(&bi_lt_a_i);
 
+    // Rcout << "  -> Reason 1: " << cx_at_a_i << " != " << grade_i << "\n";
+
     return false;
 
   }
@@ -183,6 +269,7 @@ bool is_set_preceding(SparseVector B,
     freeArray(&ci_lt_a_i);
     freeArray(&bi_lt_a_i);
 
+    // Rcout << "  -> Reason 2\n";
     return false;
 
   }
@@ -194,6 +281,7 @@ bool is_set_preceding(SparseVector B,
     freeArray(&ci_lt_a_i);
     freeArray(&bi_lt_a_i);
 
+    // Rcout << "  -> Reason 3\n";
     return false;
 
   }
@@ -207,16 +295,18 @@ bool is_set_preceding(SparseVector B,
       freeArray(&ci_lt_a_i);
       freeArray(&bi_lt_a_i);
 
+      // Rcout << "  -> Reason 4\n";
       return false;
 
     }
-    if (cx_lt_a_i.array[i] != bx_lt_a_i.array[i]) {
+    if (fabs(cx_lt_a_i.array[i] - bx_lt_a_i.array[i]) > 1.e-3) {
 
       freeArray(&cx_lt_a_i);
       freeArray(&bx_lt_a_i);
       freeArray(&ci_lt_a_i);
       freeArray(&bi_lt_a_i);
 
+      // Rcout << "  -> Reason 5\n";
       return false;
 
     }
@@ -257,7 +347,8 @@ void compute_next_closure(SparseVector A, int i,
 
     for (int grade_idx = 1; grade_idx < n_grades; grade_idx++) {
 
-      compute_direct_sum(A, a_i, grades_set[a_i][grade_idx], imax, candB);
+      bool can = compute_direct_sum(A, a_i, grades_set[a_i][grade_idx], imax, candB);
+      if (!can) continue;
 
       semantic_closure(*candB, t, LHS, RHS, &candB2);
 
@@ -291,9 +382,16 @@ void compute_next_closure(SparseVector A, int i,
 List next_closure_implications(NumericMatrix I,
                                List grades_set,
                                StringVector attrs,
+                               String connection = "standard",
+                               String name = "Zadeh",
                                bool save_concepts = true,
                                bool verbose = false) {
 
+  LogicOperator implication = get_implication(name);
+  LogicOperator tnorm = get_tnorm(name);
+  GaloisOperator intent_f = get_intent_function(connection);
+  GaloisOperator extent_f = get_extent_function(connection);
+
   int n_attributes = attrs.size();
   int n_objects = I.nrow();
 
@@ -320,12 +418,12 @@ List next_closure_implications(NumericMatrix I,
 
   SparseVector A;
   initVector(&A, n_attributes);
-  compute_closure(&A, empty, I.begin(), n_objects, n_attributes);
+  compute_closure(&A, empty, I.begin(), n_objects, n_attributes,
+                  extent_f, intent_f, tnorm, implication);
 
   SparseVector this_extent;
   initVector(&this_extent, n_objects);
 
-
   if (cardinal(A) > 0) {
 
     add_column(&LHS, empty);
@@ -349,8 +447,10 @@ List next_closure_implications(NumericMatrix I,
   if (save_concepts) {
 
     reinitVector(&this_extent);
-    compute_extent(&this_extent, A, I.begin(),
-                   n_objects, n_attributes);
+    extent_f(&this_extent, A, I.begin(),
+             n_objects, n_attributes,
+             tnorm,
+             implication);
     add_column(&concepts, A);
     add_column(&extents, this_extent);
 
@@ -404,7 +504,8 @@ List next_closure_implications(NumericMatrix I,
     }
 
     reinitVector(&B);
-    compute_closure(&B, A, I.begin(), n_objects, n_attributes);
+    compute_closure(&B, A, I.begin(), n_objects, n_attributes,
+                    extent_f, intent_f, tnorm, implication);
 
     setdifference(B, A, &rhs);
 
@@ -414,8 +515,9 @@ List next_closure_implications(NumericMatrix I,
       if (save_concepts) {
 
         reinitVector(&this_extent);
-        compute_extent(&this_extent, A, I.begin(),
-                       n_objects, n_attributes);
+        extent_f(&this_extent, A, I.begin(),
+                 n_objects, n_attributes,
+                 tnorm, implication);
 
         add_column(&concepts, A);
         add_column(&extents, this_extent);
@@ -456,7 +558,6 @@ List next_closure_implications(NumericMatrix I,
 
     if (checkInterrupt()) { // user interrupted ...
 
-
       freeVector(&A);
       freeVector(&empty);
       freeVector(&B);
@@ -515,49 +616,49 @@ List next_closure_implications(NumericMatrix I,
 
 }
 
-SparseVector compute_next_intent(SparseVector A,
-                                 NumericMatrix I,
-                                 int i,
-                                 int imax,
-                                 ListOf<NumericVector> grades_set,
-                                 int* closure_count) {
-
-
-  SparseVector candB;
-  initVector(&candB, A.length);
-
-  int n_grades = grades_set.size();
-  SparseVector candB2;
-  initVector(&candB2, A.length);
-
-  for (int a_i = i - 1; a_i >= 0; a_i--) {
-
-    n_grades = grades_set[a_i].size();
-
-    for (int grade_idx = 1; grade_idx < n_grades; grade_idx++) {
-
-      compute_direct_sum(A, a_i, grades_set[a_i][grade_idx], imax, &candB);
-
-      candB2 = compute_closure(candB, I);
-      cloneVector(&candB, candB2);
-      freeVector(&candB2);
-      (*closure_count)++;
-
-      if (is_set_preceding(A, candB, a_i, grades_set[a_i][grade_idx])) {
-
-        return candB;
-
-      }
-
-    }
-
-  }
-
-  Rprintf("Something went wrong...\n");
-
-  return candB;
-
-}
+// SparseVector compute_next_intent(SparseVector A,
+//                                  NumericMatrix I,
+//                                  int i,
+//                                  int imax,
+//                                  ListOf<NumericVector> grades_set,
+//                                  int* closure_count) {
+//
+//
+//   SparseVector candB;
+//   initVector(&candB, A.length);
+//
+//   int n_grades = grades_set.size();
+//   SparseVector candB2;
+//   initVector(&candB2, A.length);
+//
+//   for (int a_i = i - 1; a_i >= 0; a_i--) {
+//
+//     n_grades = grades_set[a_i].size();
+//
+//     for (int grade_idx = 1; grade_idx < n_grades; grade_idx++) {
+//
+//       compute_direct_sum(A, a_i, grades_set[a_i][grade_idx], imax, &candB);
+//
+//       candB2 = compute_closure(candB, I);
+//       cloneVector(&candB, candB2);
+//       freeVector(&candB2);
+//       (*closure_count)++;
+//
+//       if (is_set_preceding(A, candB, a_i, grades_set[a_i][grade_idx])) {
+//
+//         return candB;
+//
+//       }
+//
+//     }
+//
+//   }
+//
+//   Rprintf("Something went wrong...\n");
+//
+//   return candB;
+//
+// }
 
 void compute_next_intent(SparseVector* candB,
                          SparseVector A,
@@ -565,7 +666,13 @@ void compute_next_intent(SparseVector* candB,
                          int i,
                          int imax,
                          ListOf<NumericVector> grades_set,
-                         double* closure_count) {
+                         double* closure_count,
+                         StringVector attrs,
+                         GaloisOperator extent_f,
+                         GaloisOperator intent_f,
+                         LogicOperator tnorm,
+                         LogicOperator implication,
+                         bool verbose = false) {
 
 
   // SparseVector candB;
@@ -573,6 +680,7 @@ void compute_next_intent(SparseVector* candB,
   int n_objects = I.nrow();
   int n_attributes = I.ncol();
 
+  // Rcout << "** Compute next intent" << std::endl;
 
   int n_grades = grades_set.size();
   SparseVector candB2;
@@ -584,20 +692,57 @@ void compute_next_intent(SparseVector* candB,
 
     for (int grade_idx = 1; grade_idx < n_grades; grade_idx++) {
 
-      compute_direct_sum(A, a_i, grades_set[a_i][grade_idx], imax, candB);
+      bool can = compute_direct_sum(A, a_i, grades_set[a_i][grade_idx], imax, candB);
+      if (!can) continue;
+      // Rcout << "candB" << std::endl;
+      // printArray(candB->i);
+      // printArray(candB->x);
+      if (verbose) {
+
+        Rcout << "-> Testing: ";
+        printVector(*candB, attrs);
+        Rcout << "\n";
+
+      }
 
       reinitVector(&candB2);
-      compute_closure(&candB2, *candB, I.begin(), n_objects, n_attributes);
+      compute_closure(&candB2, *candB, I.begin(),
+                      n_objects, n_attributes,
+                      extent_f, intent_f,
+                      tnorm, implication);
+
+      if (verbose) {
+
+        Rcout << "-> Its closure is: ";
+        printVector(candB2, attrs);
+        Rcout << "\n";
+
+      }
 
       (*closure_count) = (*closure_count) + 1;
 
       if (is_set_preceding(A, candB2, a_i, grades_set[a_i][grade_idx])) {
 
+        if (verbose) {
+
+          Rcout << "-> It is valid!\n";
+
+        }
         // return candB;
         cloneVector(candB, candB2);
         freeVector(&candB2);
         return;
 
+      } else {
+
+        if (verbose) {
+
+          Rcout << "-> It is NOT valid!: A = ";
+          printVector(A, attrs);
+          Rcout << ", a_i = " << a_i << ", g = " << grades_set[a_i][grade_idx] << "\n";
+
+        }
+
       }
 
     }
@@ -614,6 +759,8 @@ void compute_next_intent(SparseVector* candB,
 List next_closure_concepts(NumericMatrix I,
                            ListOf<NumericVector> grades_set,
                            StringVector attrs,
+                           String connection = "standard",
+                           String name = "Zadeh",
                            bool verbose = false,
                            bool ret = true) {
 
@@ -623,6 +770,13 @@ List next_closure_concepts(NumericMatrix I,
 
   double closure_count = 0.0;
 
+  // Rcout << "Entramos" << std::endl;
+
+  LogicOperator implication = get_implication(name);
+  LogicOperator tnorm = get_tnorm(name);
+  GaloisOperator intent_f = get_intent_function(connection);
+  GaloisOperator extent_f = get_extent_function(connection);
+
   SparseVector concepts;
   SparseVector extents;
   initMatrix(&concepts, n_attributes);
@@ -633,46 +787,53 @@ List next_closure_concepts(NumericMatrix I,
   initVector(&empty, n_attributes);
   initVector(&B, n_attributes);
 
-
-  SparseVector A = compute_closure(empty, I);
+  SparseVector A;
+  initVector(&A, n_attributes);
+  compute_closure(&A, empty, I.begin(), n_objects, n_attributes,
+                  extent_f, intent_f, tnorm, implication);
   SparseVector A2;
   initVector(&A2, n_attributes);
 
   closure_count = closure_count + 1;
 
-  compute_extent(&B, A, I.begin(), n_objects, n_attributes);
+  extent_f(&B, A, I.begin(), n_objects, n_attributes, tnorm, implication);
   add_column(&concepts, A);
   add_column(&extents, B);
 
-  // if (verbose) {
-  //
-  //   Rprintf("Added concept:\n");
-  //
-  //   if (cardinal(A) > 0) {
-  //
-  //     printVector(A, attrs);
-  //
-  //   } else {
-  //
-  //     Rprintf("{}");
-  //
-  //   }
-  //
-  //   Rprintf("\n");
-  //
-  // }
+  if (verbose) {
+
+    Rprintf("Added initial concept:\n");
+
+    if (cardinal(A) > 0) {
+
+      printVector(A, attrs);
+
+    } else {
+
+      Rprintf("{}");
+
+    }
+
+    Rprintf("\n");
+
+  }
 
   // double pctg, old_pctg = 0;
 
   while ((cardinal(A) < n_attributes)){
 
+    // Rcout << "Starting iterations: " << std::endl;
     reinitVector(&A2);
     reinitVector(&B);
     compute_next_intent(&A2, A, I,
                         n_attributes,
                         n_attributes,
                         grades_set,
-                        &closure_count);
+                        &closure_count,
+                        attrs,
+                        extent_f, intent_f,
+                        tnorm, implication,
+                        verbose);
 
     // A2 = compute_next_intent(A, I,
     //                          n_attributes,
@@ -695,17 +856,20 @@ List next_closure_concepts(NumericMatrix I,
 
     // Concept
     add_column(&concepts, A2);
-    compute_extent(&B, A2, I.begin(), n_objects, n_attributes);
+    extent_f(&B, A2, I.begin(), n_objects, n_attributes, tnorm, implication);
     // B = compute_extent(A2, I);
     add_column(&extents, B);
 
-    // if (verbose) {
-    //
-    //   Rprintf("Added concept:\n");
-    //   printVector(A, attrs);
-    //   Rprintf("\n");
-    //
-    // }
+    if (verbose) {
+
+      Rprintf("Added concept:\n");
+      // Rcout << A2.i.used << std::endl;
+      // printArray(A2.i);
+      // printArray(A2.x);
+      printVector(A2, attrs);
+      Rprintf("\n");
+
+    }
 
     if (checkInterrupt()) { // user interrupted ...
 
